﻿using System;
using System.Threading;
using System.Threading.Tasks;

namespace EonaCat.HID
{
    internal class DeviceEventMonitor
    {
        public event InsertedEventHandler Inserted;
        public event RemovedEventHandler Removed;

        public delegate void InsertedEventHandler();
        public delegate void RemovedEventHandler();

        private readonly Device _device;
        private bool _wasConnected;

        public DeviceEventMonitor(Device device)
        {
            _device = device;
        }

        public void Init()
        {
#if NET20 || NET35 || NET5_0_OR_GREATER
            Task task = Task.Factory.StartNew(() => EventMonitor());
#else
             var eventMonitor = new Action(EventMonitor);
             eventMonitor.BeginInvoke(DisposeDeviceEventMonitor, eventMonitor);
#endif
        }

        private void EventMonitor()
        {
            var isConnected = _device.IsConnected;

            if (isConnected != _wasConnected)
            {
                if (isConnected && Inserted != null)
                {
                    Inserted();
                }
                else if (!isConnected && Removed != null)
                {
                    Removed();
                }

                _wasConnected = isConnected;
            }

            Thread.Sleep(500);

            if (_device.MonitorDeviceEvents)
            {
                Init();
            }
        }

        private static void DisposeDeviceEventMonitor(IAsyncResult ar)
        {
            ((Action)ar.AsyncState).EndInvoke(ar);
        }
    }
}
