﻿using System.Threading.Tasks;

namespace EonaCat.HID
{
    public class ReadDevice : Device
    {
        internal ReadDevice(string devicePath, string description = null)
            : base(devicePath, description) { }

        public DeviceData FastRead()
        {
            return FastRead(0);
        }

        public DeviceData FastRead(int timeout)
        {
            try
            {
                return ReadData(timeout);
            }
            catch
            {
                return new DeviceData(DeviceData.ReadStatus.ReadError);
            }
        }

        public void FastRead(ReadCallback callback)
        {
            FastRead(callback, 0);
        }

        public void FastRead(ReadCallback callback, int timeout)
        {
            var readDelegate = new ReadDelegate(FastRead);
            var asyncState = new StateAsync(readDelegate, callback);
            readDelegate.BeginInvoke(timeout, EndRead, asyncState);
        }

        public async Task<DeviceData> FastReadAsync(int timeout = 0)
        {
            var readDelegate = new ReadDelegate(FastRead);
#if NET20 || NET35 || NET5_0_OR_GREATER
            return await Task<DeviceData>.Factory.StartNew(() => readDelegate.Invoke(timeout));
#else
            return await Task<DeviceData>.Factory.FromAsync(readDelegate.BeginInvoke, readDelegate.EndInvoke, timeout, null);
#endif
        }

        public HidReport FastReadReport()
        {
            return FastReadReport(0);
        }

        public HidReport FastReadReport(int timeout)
        {
            return new HidReport(Capabilities.InputReportByteLength, FastRead(timeout));
        }

        public void FastReadReport(ReadReportCallback callback)
        {
            FastReadReport(callback, 0);
        }

        public void FastReadReport(ReadReportCallback callback, int timeout)
        {
            var readReportDelegate = new ReadReportDelegate(FastReadReport);
            var asyncState = new StateAsync(readReportDelegate, callback);
            readReportDelegate.BeginInvoke(timeout, EndReadReport, asyncState);
        }

        public async Task<HidReport> FastReadReportAsync(int timeout = 0)
        {
            var readReportDelegate = new ReadReportDelegate(FastReadReport);
#if NET20 || NET35 || NET5_0_OR_GREATER
            return await Task<HidReport>.Factory.StartNew(() => readReportDelegate.Invoke(timeout));
#else
            return await Task<HidReport>.Factory.FromAsync(readReportDelegate.BeginInvoke, readReportDelegate.EndInvoke, timeout, null);
#endif
        }
    }
}
