using System.Collections.Generic;
using System.Linq;

namespace EonaCat.HID
{
    public class ReadEnumerator : IEnumerator
    {
        public bool IsConnected(string devicePath)
        {
            return Devices.IsConnected(devicePath);
        }

        public IDevice GetDevice(string devicePath)
        {
            return Enumerate(devicePath).FirstOrDefault() as IDevice;
        }

        public IEnumerable<IDevice> Enumerate()
        {
            return Devices.EnumerateDevices().
                Select(d => new ReadDevice(d.Path, d.Description) as IDevice);
        }

        public IEnumerable<IDevice> Enumerate(string devicePath)
        {
            return Devices.EnumerateDevices().Where(x => x.Path == devicePath).
                Select(d => new ReadDevice(d.Path, d.Description) as IDevice);
        }

        public IEnumerable<IDevice> Enumerate(int vendorId, params int[] productIds)
        {
            return Devices.EnumerateDevices().Select(d => new ReadDevice(d.Path, d.Description)).
                Where(f => f.Attributes.VendorId == vendorId && productIds.Contains(f.Attributes.ProductId)).
                Select(d => d as IDevice);
        }

        public IEnumerable<IDevice> Enumerate(int vendorId)
        {
            return Devices.EnumerateDevices().Select(d => new ReadDevice(d.Path, d.Description)).
                Where(f => f.Attributes.VendorId == vendorId).
                Select(d => d as IDevice);
        }
    }
}